#pragma once 

#include "GameAssets.h"

// Ԃ̊NX
class State
{
protected:

  GameAssets* mAssets;
  int64_t mEnterTime;

public:

  State( GameAssets* assets )
    : mAssets( assets )
  {
    mEnterTime = mAssets->getFrame().timestamp();
  }

  virtual State* update()
  {
    return this;
  }

protected:

  // w̐A|[Y߂
  JyankenPose fingerCountToPose( Leap::Hand hand )
  {
    // LтĂw̐𐔂
    auto count = std::count_if( hand.fingers().begin(), hand.fingers().end(),
      [](const Leap::Finger& finger) { return finger.isExtended(); } );
    if ( count <= 1 ) {
      return JyankenPose::Gu;
    }
    else if ( count <= 3 ) {
      return JyankenPose::Choki;
    }
      
    return JyankenPose::Pa;
  }
};

// ACh(ҋ@)
class IdleState : public State
{
public:

  IdleState( GameAssets* assets )
    : State( assets )
  {
  }

  State* update();
};

// u񂯂v̊|s
class JyankenState : public State
{
public:

  JyankenState( GameAssets* assets )
    : State( assets )
  {
  }

  State* update();
};

// uۂv̊|ƃ|[Y̌s
class PonState : public State
{
  JyankenPose mPlayer;
  JyankenPose mComputer;
  std::list<float> mTransrates;

public:

  PonState( GameAssets* assets )
    : State( assets )
  {
    // Rs[^[̃|[Y߂
    mComputer = (JyankenPose)(Rand::randUint() % 3);
  }

  State* update();
};

// ʂ̕\s
class ResultState : public State
{
  JyankenPose mPlayer;
  JyankenPose mComputer;

public:

  ResultState( GameAssets* assets, JyankenPose player, JyankenPose computer )
    : State( assets )
    , mPlayer( player )
    , mComputer( computer )
  {
  }

  State* update();
};

State* IdleState::update()
{
  mAssets->setComputerPose( JyankenPose::Gu );
  mAssets->setText( L"oĂ" );

  // oȀԂ֑Jڂ
  auto frame = mAssets->getFrame();
  if ( frame.hands().count() == 0 ) {
    return this;
  }
  else {
    return new JyankenState( mAssets );
  }
}

State* JyankenState::update()
{
  auto hand = mAssets->getFrame().hands()[0];
  if ( hand.isValid() ) {
    mAssets->setPlayerPose( fingerCountToPose( hand ) );
  }

  mAssets->setComputerPose( JyankenPose::Gu );
  mAssets->setText( L"񂯂" );

  // 莞ԂŎ̏Ԃ֑Jڂ
  if ( (mAssets->getFrame().timestamp() - mEnterTime) < (2 * 1000 * 1000) ) {
    return this;
  }
  else {
    return new PonState( mAssets );
  }
}

State* PonState::update()
{
  mAssets->setComputerPose( mComputer );
  mAssets->setText( L"ۂ" );

  // 肪A|[Yɂ
  auto hand = mAssets->getFrame().hands()[0];
  if ( hand.isValid() ) {
    mPlayer = fingerCountToPose( hand );
    mAssets->setPlayerPose( mPlayer );
    return new ResultState( mAssets, mPlayer ,mComputer );
  }

  // 莞ԃ|[Y܂ȂAAChԂɂǂ
  if ( (mAssets->getFrame().timestamp() - mEnterTime) > (5 * 1000 * 1000) ) {
    return new IdleState( mAssets );
  }

  return this;
}

State* ResultState::update()
{
  // 𔻒肷
  mAssets->setComputerPose( mComputer );
  mAssets->setPlayerPose( mPlayer );

  if ( mComputer == mPlayer ) {
    mAssets->setText( L"" );
  }
  else if ( (mComputer == JyankenPose::Gu) && (mPlayer == JyankenPose::Pa) ) {
    mAssets->setText( L"Ȃ̏" );
  }
  else if ( (mComputer == JyankenPose::Pa) && (mPlayer == JyankenPose::Choki) ) {
    mAssets->setText( L"Ȃ̏" );
  }
  else if ( (mComputer == JyankenPose::Choki) && (mPlayer == JyankenPose::Gu) ) {
    mAssets->setText( L"Ȃ̏" );
  }
  else {
    mAssets->setText( L"Ȃ̕" );
  }

  // 莞ԂŃAChԂɖ߂
  if ( (mAssets->getFrame().timestamp() - mEnterTime) < (2 * 1000 * 1000) ) {
    return this;
  }
  else {
    return new IdleState( mAssets );
  }
}

// ԑJڂ̏
class StateMachine
{
  State* state;

public:

  StateMachine()
    : state( 0 )
  {
  }

  void setup( GameAssets* assets )
  {
    state = new IdleState( assets );
  }

  void update()
  {
    if ( state != 0 ) {
      auto nextState = state->update();
      if ( state != nextState ) {
        delete state;

        state = nextState;
      }
    }
  }
};
