#pragma once

#include "cinder/gl/gl.h"

#include "Leap.h"

using namespace ci;

class Paint
{
public:

  Paint()
    : mIs3D( false )
  {
  }

  void clear()
  {
    // OՂNA
    mCompleteStrokes.clear();
    mCurrentStrokes.clear();
  }

  void update()
  {
    mCurrentFrame = mLeap.frame();

    mPointablePoints.clear();

    // |C^uEIuWFNg̍W擾
    auto pointables = mCurrentFrame.pointables();
    for ( auto pointable : pointables ) {
      auto point = pointable.tipPosition();
      mPointablePoints.push_back( point );

      // Lȃ|C^uEIuWFNgł΁Ar̋OՂɒǉ(IDƍ킹)
      if ( isValid( pointable ) ) {
        mCurrentStrokes[pointable.id()].push_back( point );
      }
    }

    // r̋OՂ𒲂ׂāÃt[ł̃|C^uEIuWFNgȂꍇA
    // ̂IƔfAOՂƂ
    for ( auto it = mCurrentStrokes.begin(); it != mCurrentStrokes.end();  ) {
      auto pointable = mCurrentFrame.pointable( it->first );
      if ( isValid( pointable ) ) {
        ++it;
      }
      else {
        mCompleteStrokes.push_back( it->second );
        it = mCurrentStrokes.erase( it );
      }
    }
  }

  void draw()
  {
    gl::clear( Color( 0, 0, 0 ) ); 

    // ݂̃|C^uEIuWFNg̈ʒu\
    drawPoints( mPointablePoints, 15, Color( 1, 0, 0 ) );

    // OՂ\
    for ( auto strokes : mCompleteStrokes ) {
      drawLineStrip( strokes );
    }

    // r̋OՂ\
    for ( auto strokes : mCurrentStrokes ) {
      drawLineStrip( strokes.second );
    }
  }

  void set3DMode( bool is3d )
  {
    mIs3D = is3d;
  }

  bool get3DMode() const
  {
    return mIs3D;
  }

  static const int Width = 1280;
  static const int Height = 700;

private:

  bool isValid( Leap::Pointable pointable )
  {
    // Lȃ|C^(IuWFNgLŁALтĂ)
    return pointable.isValid() &&
           pointable.isExtended();
  }

  // 
  void drawLineStrip(const std::vector<Leap::Vector>& strokes)
  {
    gl::lineWidth( 10 );
    setDiffuseColor( Color::white() );
    gl::begin( GL_LINE_STRIP );

    for ( auto position : strokes ) {
      drawVertex( position );
    }

    gl::end();

    // Ɛ̕⊮
    drawPoints( strokes, 5, Color::white() );
  }

  // _
  void drawPoints( const std::vector<Leap::Vector>& points, int size, Color color )
  {
    glPointSize( size );
    setDiffuseColor( color );
    gl::begin( GL_POINTS );

    for ( auto point : points ) {
      drawVertex( point );
    }
    gl::end();
  }

  // Wݒ肷
  void drawVertex( Leap::Vector pointable )
  {
    // 3f[^ŏ
    if ( mIs3D ) {
      // 3
      gl::vertex( toVec3f( pointable ) );
    }
    // 2f[^ŏ
    else {
      // Wϊ(0-1̒l)
      auto box = mCurrentFrame.interactionBox();
      auto point = box.normalizePoint( pointable );

      // c(ʂLeap Motionɑ΂Đ)
      gl::vertex( point.x * Width, Height  - (point.y * Height) );

      // (ʂLeap Motionɑ΂Đ)
      //gl::vertex( point.x * Width, point.z * Height );
    }
  }

  // GL_LIGHT0̐Fς
  void setDiffuseColor( ci::ColorA diffuseColor )
  {
    gl::color( diffuseColor );
    glLightfv( GL_LIGHT0 , GL_DIFFUSE, diffuseColor );
  }

  // Leap SDKVectorCinderVec3fɕϊ
  Vec3f toVec3f( Leap::Vector vec )
  {
    return Vec3f( vec.x, vec.y, vec.z );
  }

private:

  bool mIs3D;

  // Leap Motion
  Leap::Controller mLeap;
  Leap::Frame mCurrentFrame;

  // `p̍W
  std::vector<std::vector<Leap::Vector>> mCompleteStrokes;
  std::map<int, std::vector<Leap::Vector>> mCurrentStrokes;
  std::vector<Leap::Vector> mPointablePoints;
};

